// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 13th May 2006
//
// Purpose:       Demo shows how to use HEL's Background Text System
//                to print text on a tile based background.
//
//                The graphic is in 4bit mode and is converted to
//                use a palette at palette bank 15.
//                See makefile for details.
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"
 
// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------

// The background number the 
// background Text System uses.
#define BGNO        0


// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// A buffer of memory which the Background Text
// System uses to manage internal states.
// It must not be modified from outside as long as
// any Background Text System is running.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_BgTextSystemMemory[HEL_SUBSYSTEM_BGTEXT_REQUIREDMEMORY];

// g_CharLUT will hold offsets to each character
// in the map data. This array basically functions
// as a lookup table. It must be 256 elements big,
// which equals the maximum number of a characters
// in ASCII (0..255).
// This buffer is only neccessary when you pass
// BGTEXT_FLAGS_GENERATELUT to hel_BgTextCreate.
// Please see documentation for details.
u16 ATTR_EWRAM g_CharLUT[256];

// That's the order of characters how they
// appear on the source graphic.
// See hel/demos/sharedmedia/charset8x8.bmp
const char CHARORDER[] =
    " BCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmn"\
    "opqrstuvwxyz0123456789?,.-;:A#^+*@"\
    "!\"%\%&/()=?|\\<>[]{}";


// -----------------------------------------------------------------------------
// Function Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_BgSetMode(0);

    // Initialize the Background Text System.
    // The g_BgTextSystemMemory array is used by the
    // Background Text System to manage internal states
    // and must not be modified from outside as long as
    // any Background Text System is running.
    hel_BgTextInit((void*)g_BgTextSystemMemory);

    // Initialize the tileset and an empty
    // mapset using regular HAMlib functions
    ham_bg[BGNO].ti = ham_InitTileSet(ResData(RES_CHARSET8X8_RAW), RES_CHARSET8X8_RAW_SIZE16, COLORS_16, TRUE);
    ham_bg[BGNO].mi = ham_InitMapEmptySet(MAP_SIZE_32X32, FALSE);
    
    // Create a Background Text System
    hel_BgTextCreate(
        BGNO,   // BgNo
        1,      // Width of one character specified in tiles
        1,      // Height of one character specified in tiles
        ResData(RES_CHARSET8X8_MAP),    // Pointer to character map
        CHARORDER,      // Pointer to array that reflects the order of characters on source graphic
        g_CharLUT,      // Pointer to buffer that receives the generated lookup table
        BGTEXT_FLAGS_GENERATELUT);  // Flags that control creation and print behaviour
        
    // Print text on background.
    // The "\n" sequence forces a new line,
    // so you can write several lines with
    // only one call to hel_BgTextPrint
    hel_BgTextPrint(BGNO,
        1,      // X
        1,      // Y
        0,      // Flags, must be zero at this time
        "Hello,\n\n"
        "I am HELs Background Text\n"
        "System. You can use me to\n"
        "print text on tile based\n"
        "backgrounds!"     // NULL-terminated string to print
        );

    // Load palette data to Vram
    hel_PalBgLoad16(ResData16(RES_BG15_PAL), 15);
    
    // Initialize and show background
    ham_InitBg(BGNO, TRUE, 0, FALSE);
    
    // Start interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);

    // infinite loop to hold
    // the application alive
    for(;;)
    {
      hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    hel_IntrAcknowledge(INT_TYPE_VBL);
}

