// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use the Object System.
//
//                If you use HEL's Object-System, you must not use any
//                object related function from HAM anymore. If you mix calls
//                to object function from both API's, HAM and HEL, the output 
//                will probably be something unexpected.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// Reserve memory for the Obj-System. This buffer
// is used to manage internal states of the Obj-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// When we create an object, HEL returns a handle
// to the created resource. The g_Handle variable
// will hold the handle returned by hel_ObjCreate,
// even though we don't need it in this demo.
THandle g_Handle;

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void);



// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Create an object!
    // You can also use hel_ObjCreate256 to create a 256 color object,
    // or hel_ObjCreate16 to create objects with 16 colors.
    g_Handle = hel_ObjCreate(
                     ResData(RES_HEART_RAW), // Pointer to source graphic
                     OBJ_SHAPE_SQUARE,       // Obj Shape
                     1,                      // Obj Size, 1 means 16x16 pixels, if Shape is set to SQUARE
                     OBJ_MODE_NORMAL,        // Obj Mode
                     COLORS_256,             // Use 256 color mode
                     0,                      // Palette number. Only neccessary in 16 color mode
                     FALSE,                  // Don't use mosaic
                     FALSE,                  // Don't flip the sprite horizontally
                     FALSE,                  // Don't flip the object vertically
                     0,                      // Priority against background. 0=higest
                     FALSE,                  // Don't make the object double-sized
                     50,                     // Destination horzintal screen coordinate in pixels
                     20                      // Destination vertical screen coordinate in pixels
                  );
                     
    
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    
    for(;;)
    {
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}

