// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to perform hblank object effects
//                using the hel_ObjGetOamSlot function as well as
//                demonstrate the usage of hel_ObjSendToBack and
//                hel_ObjBringToFront.
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"


// -----------------------------------------------------------------------------
// Defines
// -----------------------------------------------------------------------------
// X and Y position of wavy object
#define WAVY_X  (120 - 32)
#define WAVY_Y  58

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------

// Reserve memory for the Obj-System. This buffer
// is used to manage internal states of the Obj-System.
// You can achieve a little performance increase when you
// store it in IWRAM, but keep in mind, IWRAM is very limited!
// The recommended memory location is EWRAM.
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_ObjSystemBuffer[HEL_SUBSYSTEM_OBJ_REQUIREDMEMORY];

// Holds the handle of the wavy object
THandle ATTR_EWRAM g_hWavyObj;

// gets increased every frame so
// we can create a sinewave movement
// depending on this value
volatile u32 g_SinRun;

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void CreateObjects(void);
void VBLInterruptHandler(void);
void ATTR_FASTFUNC HBLInterruptHandler(void);
void PrintIntroctionText(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    ham_SetBgMode(0);
    hel_PadInit();

    ham_InitText(0);
    PrintIntroctionText();

    
    // Initialize Object-System
    hel_ObjInit(g_ObjSystemBuffer);
    
    // Create flipped objects
    CreateObjects();
                     
    // load object palette
    hel_PalObjLoad256(ResData16(RES_OBJECT_PAL));
    
    // set bg color
    TOOL_SET_BG_PAL_256(0, 130, 40, 80);
    
    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    hel_IntrStartHandler(INT_TYPE_HBL, (void*)HBLInterruptHandler);

    
    for(;;)
    {
        hel_PadCapture();
        
        if(hel_PadQuery()->Pressed.A)
        {
          hel_ObjSendToBack(hel_ObjGetFirst());
        }
        
        if(hel_PadQuery()->Pressed.B)
        {
          hel_ObjBringToFront(hel_ObjGetLast());
        }

        if(hel_PadQuery()->Pressed.Select)
        {
          hel_ObjSetVisible(g_hWavyObj, !hel_ObjIsVisible(g_hWavyObj));
        }
        
        g_SinRun++;
        
        hel_SwiVBlankIntrWait();
    }
    
    return 0;
}

// -----------------------------------------------------------------------------
// Create the three flipped objects
// -----------------------------------------------------------------------------
void CreateObjects(void)
{
  // Create a horizontally and vertically flipped object!
  g_hWavyObj = hel_ObjCreate(
                   ResData(RES_HVFLIP_RAW),// Pointer to source graphic
                   OBJ_SHAPE_SQUARE,       // Obj Shape
                   3,                      // Obj Size, 3 means 64x64 pixels, if Shape is set to SQUARE
                   OBJ_MODE_NORMAL,        // Obj Mode
                   COLORS_256,             // Use 256 color mode
                   0,                      // Palette number. Only neccessary in 16 color mode
                   FALSE,                  // Don't use mosaic
                   FALSE,                  // Don't flip the sprite horizontally
                   FALSE,                  // Don't flip the object vertically
                   0,                      // Priority against background. 0=higest
                   FALSE,                  // Don't make the object double-sized
                   WAVY_X,                 // Destination horzintal screen coordinate in pixels
                   WAVY_Y                  // Destination vertical screen coordinate in pixels
                );
                
  hel_ObjSetHFlip(g_hWavyObj, TRUE);
  hel_ObjSetVFlip(g_hWavyObj, TRUE);
  
  // Create 4 heart sprites which are positioned
  // at the 4 corners of the wavy sprite
  THandle hObj = hel_ObjCreate256(ResData(RES_HEART_RAW), OBJ_SHAPE_SQUARE, 1, OBJ_MODE_NORMAL, WAVY_X - 8, WAVY_Y - 8);  // Top left
  hel_ObjClone(hObj, WAVY_X + 48 + 8, WAVY_Y - 8);      // Top right
  hel_ObjClone(hObj, WAVY_X - 8, WAVY_Y + 64 - 8);      // Bottom left
  hel_ObjClone(hObj, WAVY_X + 48 + 8, WAVY_Y + 64 - 8); // Bottom right
}

// -----------------------------------------------------------------------------
// Prints intruction text on screen
// -----------------------------------------------------------------------------
void PrintIntroctionText(void)
{
  int i;
  
  // Clear entire text layer first
  for(i=0; i<20; i++)
    ham_DrawText(0, i, "                              ");
    
  ham_DrawText(1,1,"Press -A- to send the first");
  ham_DrawText(1,2,"object to the back.");
  
  ham_DrawText(1,17,"Press -B- to bring the last");
  ham_DrawText(1,18,"object to the front.");
}

// -----------------------------------------------------------------------------
// HBL Interupt Handler
// -----------------------------------------------------------------------------
void ATTR_FASTFUNC HBLInterruptHandler(void)
{
  // Get spot in OAM from object
  u32 OamSpot = hel_ObjGetOamSlot(g_hWavyObj);
  
  // Is it located in OAM at all?
  // OamSpot holds HANDLE_INVALID in case
  // the object does not exist or is invisible.
  if(OamSpot != HANDLE_INVALID)
  {
    s32 X = SIN((REG_VCOUNT+g_SinRun)<<2) >> 4;

    // Yes it is, so use the OamSpot to
    // set its x position directly in
    // in hardware.
    TOOL_OAM_OBJ_SET_XPOS(OamSpot, WAVY_X + X);

    // Set background color to green
    // if we got a valid OamSpot
    TOOL_SET_BG_PAL_256(0, 0, 128, 0);
  }
  else
  {
    // Set background color to red
    // if we got an invalid OamSpot
    TOOL_SET_BG_PAL_256(0, 128, 0, 0);
  }
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
   // Transmit object attributes to hardware
   hel_ObjTransmit();

   hel_IntrAcknowledge(INT_TYPE_VBL);
}

